# Generated by roxygen2: do not edit by hand

export(dominant_eigen)
export(generation_time)
export(is_leslie)
export(leslie_aggregate)
export(leslie_disaggregate)
export(leslie_dominant_eigen)
export(leslie_reproductive_values)
export(leslie_stable_age)
export(mpm_aggregate)
export(mpm_elasticity)
export(mpm_partition)
export(reproductive_values)
export(spectral_radius)
export(stable_stage)
